#ifdef GL_ES
    precision highp float;
#endif

uniform float uInnerVignetting;
uniform float uColourFactor;

varying vec2 v_texCoord;

/// <summary>
/// Computes the overlay between the source and destination colours.
/// <summary>
vec3 Overlay(vec3 src, vec3 dst)
{
	// if (dst <= �) then: 2 * src * dst
	// if (dst > �) then: 1 - 2 * (1 - dst) * (1 - src)
	return vec3((dst.x <= 0.5) ? (2.0 * src.x * dst.x) : (1.0 - 2.0 * (1.0 - dst.x) * (1.0 - src.x)),
			    (dst.y <= 0.5) ? (2.0 * src.y * dst.y) : (1.0 - 2.0 * (1.0 - dst.y) * (1.0 - src.y)),
			    (dst.z <= 0.5) ? (2.0 * src.z * dst.z) : (1.0 - 2.0 * (1.0 - dst.z) * (1.0 - src.z)));
}

void main()
{
	float SepiaValue = 1.0;
	float InnerVignetting = min(0.999, uInnerVignetting);
	float OuterVignetting = 1.0;
	float colourFactor = max(0.0, min(1.0, uColourFactor));

	// Sepia RGB value
	vec3 sepia = vec3(112.0 / 255.0, 66.0 / 255.0, 20.0 / 255.0);

	// Step 1: Desaturate color
	vec3 colour = texture2D(CC_Texture0, v_texCoord).xyz;
	float gray = (colour.x + colour.y + colour.z) / 3.0;
	vec3 grayscale = colour * 0.3 + vec3(gray) * 0.7;
	
	// Step 2: Appy sepia overlay
	vec3 finalColour = Overlay(sepia, grayscale);
	
	// Step 3: Lerp final sepia colour
	finalColour = grayscale + SepiaValue * (finalColour - grayscale);
	
	// Step 4: Lerp final colour
	finalColour = (colourFactor * colour) + ((1.0 - colourFactor) * finalColour);

	// Step 5: Apply vignetting
	// Max distance from centre to corner is ~0.7. Scale that to 1.0.
	float d = distance(vec2(0.5, 0.5), v_texCoord) * 1.414213;
	float vignetting = clamp((OuterVignetting - d) / (OuterVignetting - InnerVignetting), 0.0, 1.0);
	finalColour.xyz *= vignetting;
	
	// Apply colour
	gl_FragColor.xyz = finalColour;
	gl_FragColor.w = 1.0;
}